import java.util.*;
import java.io.*;

/**
 * Klasse Artikel: Verwaltet die Artikeldaten.
 */
public class Artikel {

    /**
     * private Hashtable daten.
     * Hier werden alle Artikeldaten abgelegt.
     */
    private Hashtable daten = new Hashtable();
    

    /**
     * In diesem Format werden die Artikeldaten abgelegt.
     */
    private class Daten
        {
        String name = null;
        double preis = 0;
    
        /**
	 * Konstruktor fr ein neues Element.
	 */
        Daten(String na, double p)
	    {
	    name = na;
	    preis = p;
	    }
        }


    /**
     *	Konstuktor: liet das angegebene File ein.
     *  Liet das durch pfad und filename angegeben File ein und Wertet den Inhalt als Artikeldaten aus.
     *  Falls der Inhalt gltig ist, wird er in einer HashTable abgelegt.
     *  @param String pfad: Gibt das Verzeichnis des einzulesenden Files an.
     *  @param String filename: Gibt den Filenamen des einzulesenden Files an.
     *  @see StringtoArtikel.
     *  @exception MyException.
     *	@see MyException.
     */
    Artikel (String pfad, String filename) throws MyException {
	String s = null;
	ReadFile file;

	// Versucht File zu ffnen. Wenn nicht mglich, Abbruch
	file = new ReadFile (pfad, filename);

	// Liet solange Artikel ein, bis keine mehr vorhanden sind    
	while ((s = file.readLine()) != null)
	    {
	    try 
		{
		// Wandelt einen String ins Artikelformat um
		StringtoArtikel (s);
		}
	    catch (MyException e)
		{
		// Fehlermeldung (ErrorDialog)
		System.out.println ("Fehler: " + e.getMessage());
		}
	    }
	file.close ();
	if (daten.isEmpty())
	    throw new MyException ("Es wurden keine Daten eingelesen - Programmabbruch");
	}

		
    /**
     * Methode: String to Artikel.
     * Versucht den bergebenen String in der Hashtable "Artikel" abzulegen.
     */
    private void StringtoArtikel (String s) throws MyException
	{
	int nummer = 0;
	String name = null;
	double preis = 0;
	
	// Initialisiert Stringteiler und definiert die Trennzeichen
	StringTokenizer T = new StringTokenizer (s, ",");

	try {
	    // berprft, ob die Parameteranzahl korrekt ist
	    if ((T.countTokens () != 3) || (s.charAt (s.length () - 1) == ','))
		throw new MyException ("Falsche Syntax: " + s + " - Datensatz ignoriert.");

	    // Liet die Artikelnummer, wenn nicht dreistellig Abbruch
	    name = T.nextToken ();
	    if (name.length() !=3)
		throw new MyException ("Ungltige Artikelnummer im angegebenen File.");
	    nummer = Integer.parseInt (name);
	    
	    if ((nummer < 100) || (nummer > 399))
		throw new MyException ("Ungltige Artikelnummer im angegebenen File.");

	    // Liet Artikelnamen, berprft, ob der Artikelname zulssig ist, und bricht bei Fehler ab
	    name = T.nextToken ();
	    if (name.length() > 20)
		throw new MyException ("Artikelbezeichnung zu lang (max. 20 Zeichen)");
    
	    // Liet den Preis
	    preis = Double.valueOf(T.nextToken()).doubleValue();
	    } 
	catch (NumberFormatException e)
	    {
	    throw new MyException (e.getMessage());
	    }
	    
	// Wenn kein Fehler bei der Umwandlung aufgetreten ist, Datensatz speichern
	daten.put (Integer.toString(nummer), new Daten (name,preis));
	}
	
	
    /**
     * Methode find (int art).
     * Sucht in Artikeldaten nach bergebener Artikelnummer und gibt den Key bei Erfolg zurck.
     * @return Daten wenn gefunden - null, wenn nicht vorhanden.
     */
    Daten find (String art)
	{
	return (Daten) daten.get (art);
	}

		
    /**
     * Wandelt einen String in einen Integer um.
     * Dieser Methode darf nur fr Strings verwendet werden, bei denen sichergestellt ist, dass sich sich umwandeln lassen.
     * @return int key.
     */
    int key (String key)
    	{
	return Integer.parseInt (key);
	}
    
    
    /**
     * Gibt die Artikelnummer des bergebenen String zurck.
     * @return int Artikelnummer.
     */
    int getArtikelNr (String s) {
	if (s == null)
	    return 0;
	for (Enumeration e = daten.keys() ; e.hasMoreElements() ;) {
	    String r = (String) e.nextElement();	
	    if (name (r).equals (s.substring (0, s.indexOf (" "))))
		return key (r);
	    }
	return -1;
	}


    /**
     * Gibt den Artikelnamen des gewnschten Artikel zurck.
     * @return den Artikelnamen.
     */
    String name (String j)
	{
	return find(j).name;
	}

    
    /**
     * Methode: preis.
     * Gibt den Preis des gewnschten Artikels zurck.
     * @return preis - der Preis des Artikels.
     */
    double preis (String j)
	{return find(j).preis;}

        
    /**
     * Methode int lenght ().
     * Liefert die Anzahl der gespeicherten Artikel.
     * @return Anzahl der Artikel.
     */
    int length ()
	{return daten.size();}	
	
	
    /**
     * Methode: berechneMenge.
     * Berechnet die Anzahl der konsumierten Artikel.
     * @param r - Die gekauften Artikel.
     * @return Vector - mit Mengen und Beschriftung.
     */
    public Vector berechneMenge (Rechnung r) {
	Vector v = new Vector ();
	for (Enumeration e = daten.keys() ; e.hasMoreElements() ;)
	    {
	    String s = (String) e.nextElement();
	    int menge = r.getArtikelMenge (key (s));
	    
	    if (menge > 0)
		v.addElement (new AusgabeDaten (menge, name (s)));
	    }
	return v;
	}


    /**
     * Methode: berechneGrpMenge.
     * Berechnet die Anzahl der konsumierten Artikelgruppen.
     * @param r - Die gekauften Artikel.
     * @return Vector - mit Mengen und Beschriftung.
     */
    public Vector berechneGrpMenge (Rechnung r) {
	String grps[] = {"Gruppe 1", "Gruppe 2", "Gruppe 3"};
	String grp = null;
	int index = 0;
	
	Vector v = new Vector ();
	for (Enumeration e = daten.keys() ; e.hasMoreElements() ;)
	    {
	    String s = (String) e.nextElement();
	    grp = grps [Integer.parseInt (s.substring (0,1)) - 1];
	    int menge = r.getArtikelMenge (key (s));
	    
	    if (menge > 0)
	    if ((index = ArtikelGrpIndex (grp, v)) >= 0) 
		{
		v.addElement (new AusgabeDaten (menge + ((AusgabeDaten)v.elementAt (index)).menge, grp));
		v.removeElementAt (index);
		}
	    else
		v.addElement (new AusgabeDaten (menge, grp));
	    }
	return v;
	}
	
	
   /**
     * Durchsucht den bergebenen Vektor nach einer Artikelgruppe und liefert deren Index zurck.
     * @param art - die gesuchte Artikelgruppe.
     * @param d - die zu untersuchender Vektor.
     * @return int - der Index der gesuchten Artikelgruppe.
     */
    private int ArtikelGrpIndex (String art, Vector d) {
	for (int i = 0; i < d.size (); i++)
	    if ((((AusgabeDaten) d.elementAt (i)).bez).equals (art))
		return i;
	return -1;
	}		
    }