import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * class MyGUI.
 * Hier wird die graphische Ausgabe erzeugt.
 * Mit Hilfe des Observer Interface werden die Rechnungsfiles berwacht.
 */
public class MyGUI extends Frame implements ItemListener, Observer, ActionListener {
    private Artikel artikelDaten;
    private Rechnung rechnungsDaten;
    private CheckboxGroup cbg1, cbg2;
    private Checkbox artikel, kassa, balken, kreis;
    private ScrollPane scrollpane;
    private OutputCanvas output;
    private Label header;
    
    final static String LBL_1 = "Verkaufte Artikel als Balkendiagramm";
    final static String LBL_2 = "Verkaufte Artikelgruppen als Kreisdiagramm";
    final static String LBL_3 = "Verkaufte Artikel pro Kassa als Balkendiagramm";
    final static String LBL_4 = "Verkaufte Artikel pro Kassa als Kreisdiagramm";
    final static int BL_VGAP = 10;
    final static int BL_HGAP = 10;    


    /**
     * Konstruktor.
     * ffnet Frame und initialisiert die Anzeige.
     * @param artikelDaten - die Artikeldaten.
     * @see Artikel.
     * @param rechnungsDaten - die Rechnungsdaten.
     * @see Rechnung.
     */
    MyGUI (Artikel artikelDaten, Rechnung rechnungsDaten) {
	// Initialisiert Frame
    	super ("Rechnungsbearbeitung");
	setSize (600, 400);
	addWindowListener(new Fensterschliesser ());
	Menue mb = new Menue (this);
	setMenuBar (mb);

	// kopiert die Referenzen
	this.artikelDaten = artikelDaten;
	this.rechnungsDaten = rechnungsDaten;

	// Setzt Leyout auf BorderLayout
	setLayout (new BorderLayout(BL_HGAP, BL_VGAP));
	
	  // Erzeugt Label:
	  header = new Label (LBL_1, Label.CENTER);
	  add ("North", header);
	  
	  // Erzeugt den Canvas + Scrollpane
	  scrollpane = new ScrollPane ();
	  output = new OutputCanvas ();
	  scrollpane.add (output);
	  add ("Center", scrollpane);
	  scrollpane.doLayout ();

	  // Setzt Scrollbareinheiten
	  scrollpane.getHAdjustable().setUnitIncrement (40);
	  scrollpane.getVAdjustable().setUnitIncrement (30);	
	
	  // Erzeugt die Radiobuttons und deren Listener
	  Panel p = new Panel ();
	    p.setLayout (new GridLayout(3, 2));
	    cbg1 = new CheckboxGroup();
	    cbg2 = new CheckboxGroup();
	    p.add (new Label ("Was anzeigen?", Label.LEFT));
	    p.add (new Label ("Wie anzeigen?", Label.LEFT));
	    artikel = new Checkbox("Artikel", cbg1, true);
	    artikel.addItemListener (this);
	    p.add (artikel);
	    balken = new Checkbox("Balkendiagramm", cbg2, true);
	    balken.addItemListener (this);
	    p.add (balken);
	    kassa = new Checkbox("Kassen", cbg1, false);
	    kassa.addItemListener (this);
	    p.add (kassa);
	    kreis = new Checkbox("Kreisdiagramm", cbg2, false);
	    kreis.addItemListener (this);
	    p.add (kreis);
	  add ("South", p);

	// Zeigt fertigen Frame an	
	setVisible (true);
	}
    
    
    /**
     * Methode paint.
     * fhrt keine Anweisung aus.
     */
    public void paint (Graphics g)
	{}
    
    
    /**
     * Methode: itemStateChanged.
     * Wertet Vernderungen in der CheckBoxGroup aus & veranlasst ein Neuzeichnen des Canvas.
     */
    public void itemStateChanged (ItemEvent e)
	{
        if ((e.getSource() == kassa) && output.artikel)
		output.artikel = false;
	else if ((e.getSource () == artikel) && !output.artikel)
		output.artikel = true;
        else if ((e.getSource () == kreis) && output.balken)
		output.balken = false;
	else if ((e.getSource () == balken) && !output.balken)
		output.balken = true;
	
	output.repaint ();
	}
	

    /**
     * Wertet die ActionEvents des Mens aus.
     */
    public void actionPerformed (ActionEvent e) {
	String s = e.getActionCommand();
	
	// Dialog zum Rechnung eingeben aufrufen
	if (s.equals("Rechnung Eingeben")) {
	    DialogNeu dNeu = new DialogNeu (this, "Rechnung Bearbeiten", false, rechnungsDaten, artikelDaten);
	    dNeu.setVisible(true);	
	    }
	    
	// Dialog zum Rechnung ndern aufrufen
	else if (s.equals("Rechnung ndern")) {
	    DialogBearbeiten dEdit = new DialogBearbeiten (this, "Rechnung Bearbeiten", false, rechnungsDaten, artikelDaten);
	    rechnungsDaten.addObserver (dEdit);
	    dEdit.setVisible(true);	
	    }
	    
	// Dialog zum Rechnung lschen aufrufen
	else if (s.equals("Rechnung Lschen")) {
	    DialogLoeschen dLschen = new DialogLoeschen (this, "Rechnung Lschen", false, rechnungsDaten);
	    dLschen.setVisible(true);	
	    }
	    
	// Programm beenden
	else if (s.equals("Beenden")) System.exit(0);
	}	


    /**
     * Klasse: OutputCanvas.
     * Die Zeichenflche. Die Abmessungen des Canvas werden durch die auszugebenden Strings definiert.
     * Der Canvas ist aber mindestens so gro, wie der geffnete Frame.
     */
    private class OutputCanvas extends Canvas
	{
	public boolean balken = true, artikel = true;
	public int anz = 0;
	
	/**
	 * Konstruktor.
	 * Hier wird die Gre des Canvas berechnet & gesetzt
	 */
	OutputCanvas ()
	    {
	    Dimension groesse;
	    groesse = scrollpane.getViewportSize ();
	    setSize (groesse.width, groesse.height);
	    }
	
	/**
	 * Methode: paint.
	 * Hier wird das gewnschte Diagramm ausgegeben.
	 **/
	public void paint (Graphics g)
	    {
	    Vector h = new Vector ();
	    
	    // Auswertung, welches Diagramm gezeichnet werden soll
	    if (artikel && balken) {
		header.setText (LBL_1);
		drawBalkenDiagramm (g, artikelDaten.berechneMenge (rechnungsDaten));
		}
	    else if (artikel && !balken) {
		header.setText (LBL_2);
		drawKreisDiagramm (g, artikelDaten.berechneGrpMenge (rechnungsDaten));
		}
	    else if (!artikel && balken) {
		header.setText (LBL_3);
		drawBalkenDiagramm (g, rechnungsDaten.berechneMenge ());
		}
	    else if (!artikel && !balken) {
		header.setText (LBL_4);
		drawKreisDiagramm (g, rechnungsDaten.berechneMenge ());
		}
	    }
	
	
	/**
	 * Zeichnet Balkendiagramm.
	 * @param g - die Graphikinstanz.
	 * @param h - die Auszugebenden Daten in einem Vektor.
	 */
	private void drawBalkenDiagramm (Graphics g, Vector h) {
	    int hspace = 0, xbez = 20, ybez, maxX, maxY, breite = 20;
	    double fact = 0.0;
	    Dimension groesse;
	    MyColors m = new MyColors ();
	    FontMetrics fm;
	    
	    groesse = scrollpane.getViewportSize ();
	    maxX = groesse.width * 7 / 10;
	    maxY = groesse.height * 7 / 10;
	    ybez = groesse.height / 10;	    
	    g.setFont (fitFont (ybez));
	    fm = g.getFontMetrics ();
	    
	    // Berechnet die max. Breite einer Bezeichnung + hchsten Balken
	    for (int i = 0; i < h.size (); i++)
		{
		if (fm.stringWidth(((AusgabeDaten)h.elementAt (i)).bez) + 10 > hspace)
		    hspace = fm.stringWidth(((AusgabeDaten)h.elementAt (i)).bez) + 10;
		if (fm.stringWidth("" + ((AusgabeDaten)h.elementAt (i)).menge) + 10 > hspace)
		    hspace = fm.stringWidth("" + ((AusgabeDaten)h.elementAt (i)).menge) + 10;
		if (fact < (1.0*((AusgabeDaten)h.elementAt (i)).menge/(maxY)))
		    fact = 1.0*((AusgabeDaten)h.elementAt (i)).menge/(maxY);
		}
	    
	    if (fact == 0)
		fact = 1;
	    if (hspace <= breite)
		hspace = breite * 2;
	    if ((h.size() * hspace + 2 * xbez) < groesse.width)
		output.setSize (groesse.width, groesse.height);
	    else
		// Passt den Canvas an das auszugebende Diagramm an
		output.setSize (h.size() * hspace + 2 * xbez, groesse.height);
		
	    scrollpane.doLayout ();
						    		
	    // Gibt das Diagramm aus    	    
	    for (int i = 0; i < h.size (); i++) {
		g.setColor (Color.black);
		drawStringZentriert (g, ((AusgabeDaten)h.elementAt (i)).bez, xbez + hspace*i, maxY + ybez*5/2, hspace);
		drawStringZentriert (g, "" + ((AusgabeDaten)h.elementAt (i)).menge, xbez + hspace*i, maxY + ybez - (int)Math.round(((AusgabeDaten)h.elementAt (i)).menge / fact), hspace);
		g.setColor (m.getNext ());
		g.fillRect(xbez + hspace*i + hspace/2 - breite/2, maxY + ybez*3/2 - (int)Math.round(((AusgabeDaten)h.elementAt (i)).menge / fact), breite, (int)Math.round(((AusgabeDaten)h.elementAt (i)).menge / fact));
		}
	    }
    
    
	/**
	 * Zeichnet Kreisdiagramm.
	 * @param g - die Graphikinstanz.
	 * @param h - die Auszugebenden Daten in einem Vektor.
	 */
	private void drawKreisDiagramm (Graphics g, Vector h) {
	    int hspace = 0, xbez = 20, ybez, maxX, maxY, breite = 20;
	    int ges = 0, start = 0, radius;
	    Dimension groesse;
	    MyColors m = new MyColors ();
	    FontMetrics fm;
	    
	    groesse = scrollpane.getViewportSize ();
	    maxX = groesse.width * 8 / 10;
	    maxY = groesse.height * 8 / 10;
	    radius = Math.min (maxX, maxY)/2;
	    ybez = maxY;	    
	    g.setFont (fitFont (maxY/8));
	    fm = g.getFontMetrics ();
	    
	    // Berechnet die Gesamtanzahl der Artikel
	    for (int i = 0; i < h.size (); i++)
		ges += ((AusgabeDaten)h.elementAt (i)).menge;
	    
	    if (hspace <= breite)
		hspace = breite * 2;
		
	    // Passt den Canvas an das auszugebende Diagramm an
	    output.setSize (groesse.width, groesse.height);
	    scrollpane.doLayout ();
	    
	    // Gibt das Diagramm aus    	    
	    for (int i = 0; i < h.size (); i++) {
		int bogen = (int) Math.round (360.0 * ((AusgabeDaten)h.elementAt (i)).menge / ges);
		
//		int texty = (int) Math.round (radius * Math.asin (start + bogen/2));
//		int textx = (int) Math.round (texty * Math.atan (start + bogen/2));
//		System.out.println (textx + " " + texty + " " + Math.asin (0.1));
/*		g.setColor (m.getNext ());
		g.fillArc (maxX/9, maxY/9, 200, 200, 0, 180);*/
//		g.setColor (Color.black);
//		drawStringZentriert (g, ((AusgabeDaten)h.elementAt (i)).bez, groesse.width/2 + textx, groesse.height/2 + texty, hspace);
		g.setColor (m.getNext ());
		g.fillArc (groesse.width/2 - radius, groesse.height/2 - radius, radius*2, radius*2, start, bogen);
		start += bogen;
		}
	    g.setColor (Color.black);		
	    g.drawOval (groesse.width/2 - radius, groesse.height/2 - radius, radius*2, radius*2);
	    }
	}
    
    
    /**
     * Berechnet die optimale Schriftgrsse.
     */
    private Font fitFont (int max) {
	FontMetrics fm;
	Font font = null;
	
	int groesse = 20;
	while (groesse > 6) {
	    font = new Font ("Serif", Font.PLAIN, groesse);
	    fm = getFontMetrics (font);
	    if (fm.getHeight () + 5 < max) break;
	    groesse--;
	    }
	return font;
	}
	
	
    /**
     * Gibt einen String zentriert aus.
     */
    private void drawStringZentriert (Graphics g, String s, int x, int y, int m) {
	FontMetrics fm = g.getFontMetrics ();
	int w = fm.stringWidth (s);
	
	g.drawString (s, x + (int)m/2 - (int)w/2, y);
	}


    /**
     * Methode: Update.
     * Fhrt bei einer Vernderung des zu berwachenden Objektes die Repaintmethode des Canvas aus.
     * @param o - das berwachte Objekt.
     * @param arg - die Vernderung.
     */
    public void update(Observable o, Object arg) {
	output.repaint ();
	}
	
	
    /**
     * Klasse: Fensterschliesser.
     * Implementiert das windowClosing Event zum Schlieen des Frames.
     */
    class Fensterschliesser extends WindowAdapter
	{
	public void windowClosing (WindowEvent e)
	    {System.exit (0);}
	}	
    }