import java.util.*;
import java.io.*;

/**
 * Klasse Rechnung.
 * Verwaltet und beinhaltet die Rechnungsdaten.
 */
class Rechnung extends Observable {
    private int aktuelleRechnung = 0;
    private int aktuellesElement = 0;

    /**
     * Vektor daten.
     * Hier werden die Rechnungselemente gespeichert.
     */
    private Vector daten = new Vector ();
    

    /**
     * Klasse RechnungsDaten.
     * In diesem Format wird eine gesamte Rechnung abgelegt.
     */
    public class RechnungsDaten
	{
	int rechnungsNr, kassa;
	Vector elements;
	RechnungsDaten (int r, int k, Vector e)
	    {
	    rechnungsNr = r;
	    kassa = k;
	    elements = e;
	    }
	}
	
	
    /**
     * Konstruktor.
     * Liet alle Rechnungsfiles aus dem angegebenen Verzeichnis ein.
     * @param String pfad: Verzeichnis aus dem die Rechnungsfiles gelesen werden sollen.
     * @exception MyException.
     * @see MyException.
     */
    Rechnung (String pfad) throws MyException {
	// ffnet Directory
	File file = new File (pfad);
		
	// Liet alles Filenamen im angegebenen Directory ein
	String f[] = file.list ();
	
	for (int i = 0; i < f.length; i++)
	    if ((f[i].length() > 14) && (f[i].substring (0, 8).equalsIgnoreCase ("rechnung")) && (f[i].substring (f[i].length() - 4, f[i].length()).equalsIgnoreCase (".txt")))
		{
		StringTokenizer st = new StringTokenizer (f[i], ".");
		if ((st.countTokens() == 3) && (isValidNumber (st.nextToken())) && (isValidKassa (st.nextToken())))
		    ReadRechnung (pfad, f[i]);
		}
	
	Observable o = new Observable ();
	}


    /**
     * Methode: load.
     * Liet die Rechnungen aus angegebenem File ein.
     * @param pfad - gibt den Pfad an, in dem sich das Rechnungsfile befindet.
     * @param filename - gibt den Filenamen an.
     * @exception MyException.
     * @see MyException.
     */
    void ReadRechnung (String pfad, String filename) throws MyException
	{
	String s = null;
	int artind = 0;
	ReadFile file;
	boolean err;
	Vector v = new Vector ();
		
	// ffnet File
	file = new ReadFile (pfad, filename);

	// Liet bis zum Fileende ein, oder bricht bei mehr als 10 Rechnungsstzen ab
	while ((s = file.readLine()) != null)
	    {
	    err = false;
	    ElementeDaten h = null;
	    // Wandelt Sting in Rechnungsdatensatz um
	    try {h = StringtoDaten (s);}
	    catch (MyException e) {
		System.out.println (e.getMessage ());
		err = true;}

	    // berprft, ob dieser Artikel in der Rechnung schon vorkommt
	    //  ja: ergnzt den alten Rechnungssatz
	    //  nein: legt neuen Rechnungssatz an
	    if (!err)
		if ((artind = artikelIndex (h.artikel, v)) >= 0)
		    {
		    h.menge += ((ElementeDaten)v.elementAt (artind)).menge;
		    v.addElement (h);
		    v.removeElementAt (artind);
		    }
		else
		    v.addElement (h);
	    }
	if (!v.isEmpty()) 
	    daten.addElement (new RechnungsDaten (Integer.parseInt(filename.substring (8, filename.indexOf ("."))), Integer.parseInt (filename.substring (filename.indexOf (".")+1, filename.length () - 4)), v));
	}    


    /**
     * Methode StringtoRechnung.
     * Versucht den bergebenen String in das Datenformat umzuwandeln.
     * @param s - der umzuwandelnde String.
     * @return Daten - Rechnungsdatensatz.
     */
    private ElementeDaten StringtoDaten (String s) throws MyException
	{
	int artikel = 0, stck = 0, artnr = 0;
	String help = null;
	
	// Initialisiert Stringteiler und definiert die Trennzeichen
	StringTokenizer st = new StringTokenizer (s, ",");

	try {
	    if ((st.countTokens () != 2) || (s.charAt (s.length () - 1) == ','))
		throw new MyException ("Falsche Syntax: " + s + " - Datensatz ignoriert.");
	    artikel = Integer.parseInt (st.nextToken());
	    stck = Integer.parseInt (st.nextToken());
	    } 
	catch (NumberFormatException e)
	    {
	    throw new MyException ("Fehler im Datensatz: " + s + " - Datensatz ignoriert.");
	    }
	return new ElementeDaten (artikel, stck);
	}		 	 	 
	 
	             
    /**
     * Methode: isValidNumber (String s).
     * berprft, ob der Filename eine gltige Rechnungsnummer enthlt.
     * @param s - Erster Teil des Filenamens.
     * @return true wenn gltig.
     */
    private boolean isValidNumber (String s) {
        try {if (Integer.parseInt (s.substring (8, s.length())) <= 0)
		return false;}
	catch (NumberFormatException e) {return false;}
	return true;
	}
	
	
    /**
     * Methode: isValidKassa (String s).
     * berprft, ob der Filename eine gltige Kassennummer enthlt.
     * @param s - Zweiter Teil des Filenamens.
     * @return true wenn gltig.
     */
    private boolean isValidKassa (String s) {
        try {if (Integer.parseInt (s) <= 0)
		return false;}
	catch (NumberFormatException e) {return false;}
	return true;
	}
	

    /**
     * Methode: berechneMenge.
     * Berechnet die Anzahl der konsumierten Artikel pro Kassa.
     * @param r - Die gekauften Artikel.
     * @return Vector - mit Mengen und Beschriftung.
     */
    Vector berechneMenge () {
	Vector v = new Vector ();
	int ind = 0;
		
	for (int i = 0; i < daten.size (); i++) {
	    ind = kassaIndex (((RechnungsDaten)daten.elementAt (i)).kassa, v);
	    if (ind >= 0) 
		    {
		    v.addElement (new AusgabeDaten (VerkaufteArtikelproKassa (((RechnungsDaten)daten.elementAt (i)).elements) + ((AusgabeDaten)v.elementAt (ind)).menge,
		    "" + ((RechnungsDaten)daten.elementAt (i)).kassa));
		    v.removeElementAt (ind);
		    }
	    else
	        v.addElement (new AusgabeDaten (VerkaufteArtikelproKassa (((RechnungsDaten)daten.elementAt (i)).elements), "" + ((RechnungsDaten)daten.elementAt (i)).kassa));
	    }
	return v;
	}    

    
    /**
     * Berechnet die Gesamtmenge der Artikel in einer Rechnung.
     * @param d - die Rechnungen.
     * @return Verkauften Artikel in einer Rechnung.
     */
    private int VerkaufteArtikelproKassa (Vector d) {
	int anz = 0;
	for (int i = 0; i < d.size (); i++)
	    anz += ((ElementeDaten) d.elementAt (i)).menge;
	return anz;
	}
	    
    
    /**
     * Durchsucht den bergebenen Vektor nach einer bestimmten Kassa und liefert deren Index zurck.
     * @param art - der gesuchte Artikel.
     * @param d - die zu untersuchender Vektor.
     * @return int - der Index der gesuchten Kassa.
     */
    private int kassaIndex (int kassa, Vector d) {
	for (int i = 0; i < d.size (); i++)
	    if ((((AusgabeDaten) d.elementAt (i)).bez).equals ("" + kassa))
		return i;
	return -1;
	}	
    
    
    /**
     * Durchsucht eine Rechnung nach einem bestimmten Artikel und liefert dessen Index zurck.
     * @param art - der gesuchte Artikel.
     * @param d - die zu untersuchende Rechnung.
     * @return int - der Index des gesuchten Artikels.
     */
    private int artikelIndex (int art, Vector d) {
	for (int i = 0; i < d.size (); i++)
	    if (((ElementeDaten) d.elementAt (i)).artikel == art)
		return i;
	return -1;
	}	
    
    
    /**
     * Berechnet die Gesamtmenge eines Artikels in einer Rechnung.
     * @param art - der gesuchte Artikel.
     * @param d - die zu untersuchende Rechnung.
     * @return int - die Menge des gesuchten Artikels.
     */
    private int CountArtOccurence (int art, Vector d) {
	int i;
	if ((i = artikelIndex (art, d)) >= 0)
	    return ((ElementeDaten) d.elementAt (i)).menge;
	return 0;
	}
    
    
    /**
     * Berechnet die Gesamtmenge eines Artikels in allen Rechnungen.
     * @param art - der gesuchte Artikel.
     * @return occ - die Menge des gesuchten Artikels.
     */
    public int getArtikelMenge (int art) {
	int occ = 0;
	for (int i = 0; i < daten.size (); i++)	
	    occ += CountArtOccurence (art, ((RechnungsDaten)daten.elementAt (i)).elements);
	return occ;
	}
	
    
    /**
     * Liefert eine Referenz auf den Vektor mit den gespeicherten Daten.
     */
    public Vector getElements () {
	return daten;
	}
    
    
    /**
     * Liefert eine Referenz auf den Vektor mit den gespeicherten Daten.
     * @param index - der Index der gesuchte Rechnungsdaten.
     * @return Vector - mit den Rechnungsdaten.
     */
    public Vector getElements (int index) {
	return ((RechnungsDaten) daten.elementAt (index)).elements;
	}
	
	
    /**
     * Liefert eine Referenz auf das Element mit den gespeicherten Daten.
     * @param nr - die Artikelnummer des gesuchten Elements.
     */
    public ElementeDaten getElement (int nr) {
	Vector e = getElements (aktuelleRechnung);
	for (int i = 0; i < e.size (); i++)
	    {
	    if (nr == ((ElementeDaten) e.elementAt (i)).artikel)
		return ((ElementeDaten) e.elementAt (i));
	    }
	return null;
	}
		
	
	
    /**
     * Erhht aktuellesElement, wenn mglich.
     */ 
    public void nextElement () {
	if (elementsSize () > aktuellesElement)
	    aktuellesElement++;
	}
	
    
    /**
     * Liefert die Menge des aktuellen Elements zurck.
     * @returns int - Menge.
     */
    public int elementMenge () {
	Vector e = getElements (aktuelleRechnung);
	return ((ElementeDaten)e.elementAt (aktuellesElement)).menge;
	}
	
    
    /**
     * Liefert die Artikelnummer des aktuellen Elements zurck.
     * @return int - Artikelnummer.
     */
    public int elementArtikel () {
	Vector e = getElements (aktuelleRechnung);
	return ((ElementeDaten)e.elementAt (aktuellesElement)).artikel;
	}
	
	
    /**
     * Setzt die aktuelle Rechnung und das aktuelle Element auf 0 zurck.
     */
    public void resetRechnung () {
	aktuelleRechnung = 0;
	aktuellesElement = 0;
	}
    
    
    /**
     * Sucht nach einer Rechnung und setzt, wenn gefunden aktuelleRechnung auf die gesuchte.
     * @param s - die gesuchte Rechung.
     */
    public void findRechnung (String s) {
	for (int i = 0; i < daten.size (); i++)
	    if (s.substring (8,s.indexOf (".")).equals ("" + ((RechnungsDaten)daten.elementAt (i)).rechnungsNr))
		aktuelleRechnung = i;
	}
	
    
    /**
     * Liefert die Anzahl der Element der aktuellen Rechnung zurck.
     * @returns int - Menge der Elemente.
     */
    public int elementsSize () {
	return ((RechnungsDaten)daten.elementAt (aktuelleRechnung)).elements.size ();
	}
    
	
    /**
     * Liefert die Anzahl der Rechnungen zurck.
     * @returns int - Anzahl der Rechnungen.
     */
    public int RechnungsSize () {
	return daten.size ();   
	}
    
    
    /**
     * Liefert den Filename der gesuchten Rechnung.
     * @param i - die gesucht Rechnung.
     * @return Filenamen der Rechnung.
     */
    public String getName (int i) {
	return "Rechnung" + ((RechnungsDaten)daten.elementAt (i)).rechnungsNr + "." + ((RechnungsDaten)daten.elementAt (i)).kassa + ".txt";
	}

        
    /**
     * Lscht eine Rechnung.
     * @param s - die zu lschende Rechnung.
     */
    public void removeRechnung (String s) {
	for (int i = 0; i < daten.size (); i++)
	    if (s.substring (8,s.indexOf (".")).equals ("" + ((RechnungsDaten)daten.elementAt (i)).rechnungsNr))
		daten.removeElementAt (i);

	// Informiert Observer	
	setChanged();
	notifyObservers();
	}


    /**
     * Lscht ein Artikelelement aus der aktuellen Rechnung.
     * @param s - der zu lschende Artikel.
     * @param a - Instanz auf die artikelDaten
     */
    public void removeElement (String s, Artikel a) {
	Vector e = getElements (aktuelleRechnung);
	for (int i = 0; i < e.size (); i++)
	    if (s.substring (0,s.indexOf (" ")).equals (a.name ("" + ((ElementeDaten)e.elementAt (i)).artikel)))
		e.removeElementAt (i);
		
	// Informiert Observer	
	setChanged();
	notifyObservers();
	}
	
    
    /**
     * berprft, ob eine Rechnung schon Existiert.
     * @param nr - die zu vergleichende Nummer.
     * @returns true, wenn die Rechnung bereits vorhanden ist.
     */
    public boolean RechnungExists (int nr) {
	for (int i = 0; i < daten.size (); i++)
	    {
	    System.out.println (nr + " " + ((RechnungsDaten)daten.elementAt (i)).rechnungsNr);
	    if (nr == ((RechnungsDaten)daten.elementAt (i)).rechnungsNr)
		return true;
	    }
	return false;
	}
    
    
    /**
     * Fgt eine neue Rechnung hinzu.
     * @param nr - die anzulegende Rechnung.
     * @param kassa - die Kasse.
     */
    public boolean addRechnung (int nr, int kassa, Vector v) {
	if (v.size () > 0) {
	    daten.addElement (new RechnungsDaten (nr, kassa, v));
	    
	    // Informiert Observer	
	    setChanged();
	    notifyObservers();
	    return true;
	    }
	return false;
	}

		
    /**
     * Setzt die Artikelmenge des angegebenen Artikels, der aktuellen Rechnung.
     * @param menge - die neue Artikelmenge
     * @param artnr - die Artikelnummer des zu verndernden Artikels
     */
    public void setMenge (int menge, int artnr) {
	ElementeDaten el = getElement (artnr);
	
	el.menge = menge;
	
	// Informiert Observer	
	setChanged();
	notifyObservers();
	}
    }